@@ -53,6 +53,8 @@ DEFINE_TEST(test_read_format_cpio_bin_gz)
 	assertEqualInt(ARCHIVE_OK, archive_read_next_header(a, &ae));
 	assertEqualInt(archive_filter_code(a, 0),
 	    ARCHIVE_FILTER_GZIP);
+	assertEqualInt(archive_entry_is_encrypted(ae), 0);
+	assertEqualIntA(a, archive_read_has_encrypted_entries(a), ARCHIVE_READ_FORMAT_ENCRYPTION_UNSUPPORTED);
 	assertEqualInt(archive_format(a), ARCHIVE_FORMAT_CPIO_BIN_LE);
 	assertEqualInt(ARCHIVE_OK, archive_read_close(a));
 	assertEqualInt(ARCHIVE_OK, archive_read_free(a));
