@@ -30,7 +30,9 @@ static unsigned char tmp[1000];
 static unsigned char  buff[10000];
 size_t data_sizes[] = {0, 5, 511, 512, 513};
 
-void
+static void verify_read_positions(struct archive *a);
+
+static void
 verify_read_positions(struct archive *a)
 {
 	struct archive_entry *ae;
@@ -103,7 +105,7 @@ DEFINE_TEST(test_read_position)
 	/* Read the archive back without a skip function. */
 	assert(NULL != (a = archive_read_new()));
 	assertA(0 == archive_read_support_format_tar(a));
-	assertA(0 == read_open_memory2(a, buff, sizeof(buff), 512));
+	assertA(0 == read_open_memory_minimal(a, buff, sizeof(buff), 512));
 	verify_read_positions(a);
 	archive_read_free(a);
 
